@@ -32,8 +32,8 @@ typedef struct _GsfInfileStdio GsfInfileStdio;
 #define GSF_INFILE_STDIO(o)          (G_TYPE_CHECK_INSTANCE_CAST ((o), GSF_INFILE_STDIO_TYPE, GsfInfileStdio))
 #define GSF_IS_INFILE_STDIO(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o), GSF_INFILE_STDIO_TYPE))
 
-GType		gsf_infile_stdio_get_type (void);
-GsfInfileStdio *gsf_infile_stdio_new (char const *root, GError **err);
+GType	   gsf_infile_stdio_get_type (void);
+GsfInfile *gsf_infile_stdio_new      (char const *root, GError **err);
 
 G_END_DECLS
 
